/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <vte/vte.h>
#include <vte/reaper.h>
#include <string.h>

#include "gpt.h"
#include "auth.h"
#include "misc.h"

extern GptApp *the_app;

static void gpt_auth_window_class_init  (GptAuthWindowClass *klass);
static void gpt_auth_window_init        (GptAuthWindow *pw);

static void on_btn_cancel_clicked_cb    (GtkWidget *widget,
                                         GptAuthWindow *aw);
static void on_btn_ok_clicked_cb        (GtkWidget *widget,
                                         GptAuthWindow *aw);
static void on_child_exited_cb          (VteReaper *reaper,
                                         gint child_pid,
                                         gint retval,
                                         GptAuthWindow *aw);


static GtkWindowClass *parent_class = NULL;

gboolean
need_su (GtkWindow *w)
{
  GtkWidget *dlg;
  GtkWidget *dlg_root;
  GtkWidget *hbox;
  GtkWidget *icon;
  GtkWidget *label;
  GtkWidget *action_area;
  GtkWidget *btn_accept;
  GtkWidget *btn_cancel;
  GtkWidget *alignment;
  GtkWidget *hbox2;
  GtkWidget *icon2;
  GtkWidget *label2;

  dlg = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlg), "Question");
  gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
  gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dlg), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (dlg), TRUE);
  gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
  gtk_window_set_type_hint (GTK_WINDOW (dlg), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_transient_for (GTK_WINDOW (dlg), w);
	
  dlg_root = GTK_DIALOG (dlg)->vbox;
	
  hbox = gtk_hbox_new (FALSE, 12);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (dlg_root), hbox, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
	
  icon = gtk_image_new_from_stock ("gtk-dialog-warning",
                                   GTK_ICON_SIZE_DIALOG);
  gtk_widget_show (icon);
  gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, TRUE, 0);
	
  label = gtk_label_new ("You must be superuser (root) to be able to install and uninstall packages. Would you like to enter root password to enable full functionality?");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
	
  action_area = GTK_DIALOG (dlg)->action_area;
  gtk_widget_show (action_area);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area),
                             GTK_BUTTONBOX_END);
	
  btn_cancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (btn_cancel);
  gtk_dialog_add_action_widget (GTK_DIALOG (dlg),
                                btn_cancel,
                                GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (btn_cancel,
                        GTK_CAN_DEFAULT | GTK_HAS_DEFAULT);
	
  btn_accept = gtk_button_new ();
  gtk_widget_show (btn_accept);
  gtk_dialog_add_action_widget (GTK_DIALOG (dlg),
                                btn_accept,
                                GTK_RESPONSE_ACCEPT);
  GTK_WIDGET_SET_FLAGS (btn_accept, GTK_CAN_DEFAULT);
	
  alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment);
  gtk_container_add (GTK_CONTAINER (btn_accept), alignment);
	
  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment), hbox2);
	
  icon2 = gtk_image_new_from_stock ("gtk-dialog-authentication",
                                    GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (icon2);
  gtk_box_pack_start (GTK_BOX (hbox2),
                      icon2,
                      FALSE, FALSE, 0);
	
  label2 = gtk_label_new_with_mnemonic ("Enter root password");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox2),
                      label2,
                      FALSE, FALSE, 0);
	
  if (gtk_dialog_run (GTK_DIALOG (dlg)) != GTK_RESPONSE_ACCEPT)
    {
      gtk_widget_destroy (dlg);
      return FALSE;
    }
  else
    {	
      gtk_widget_destroy (dlg);
      return TRUE;
  }
}

/******************************************************************************/

GType
gpt_auth_window_get_type (void)
{
  static GType type = 0;

  if (type == 0)
    {
      static const GTypeInfo info =
        {
          sizeof (GptAuthWindowClass),
          NULL,   /* base_init */
          NULL,   /* base_finalize */
          (GClassInitFunc) gpt_auth_window_class_init,   /* class_init */
          NULL,   /* class_finalize */
          NULL,   /* class_data */
          sizeof (GptAuthWindow),
          0,      /* n_preallocs */
          (GInstanceInitFunc) gpt_auth_window_init    /* instance_init */
        };
		
      type = g_type_register_static (GTK_TYPE_WINDOW, "GptAuthWindow", &info, 0);
    }
  return type;
}

static void
gpt_auth_window_class_init (GptAuthWindowClass *klass)
{
  parent_class = (GtkWindowClass *) g_type_class_peek_parent (klass);
}

static void
gpt_auth_window_init (GptAuthWindow *aw)
{
  GtkWidget *dlg_root;
  GtkWidget *hbox;
  GtkWidget *icon;
  GtkWidget *vbox;
  GtkWidget *label;
  GtkWidget *action_area;
	
  aw->priv = g_new0 (GptAuthWindowPriv, 1);

  gtk_container_set_border_width (GTK_CONTAINER (aw), 12);
  gtk_window_set_title (GTK_WINDOW (aw), "Authentication");
  gtk_window_set_modal (GTK_WINDOW (aw), TRUE);
  gtk_window_set_resizable (GTK_WINDOW (aw), FALSE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (aw), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (aw), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (aw), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_transient_for (GTK_WINDOW (aw), GTK_WINDOW (the_app->main_window));

  /* info/buttons VBox */
  dlg_root = gtk_vbox_new (FALSE, 13);
  gtk_widget_show (dlg_root);
  gtk_container_add (GTK_CONTAINER (aw), dlg_root);

  /* icon/label+entry HBox */
  hbox = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (dlg_root), hbox, TRUE, TRUE, 0);

  /* icon */
  icon = gtk_image_new_from_stock ("gtk-dialog-authentication", GTK_ICON_SIZE_DIALOG);
  gtk_widget_show (icon);
  gtk_box_pack_start (GTK_BOX (hbox), icon, TRUE, TRUE, 0);

  /* label/entry VBox */
  vbox = gtk_vbox_new (FALSE, 12);
  gtk_widget_show (vbox);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 3);

  /* label */
  label = gtk_label_new ("Please enter superuser (root) password:");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

  /* password entry */
  aw->priv->entry = gtk_entry_new ();
  gtk_widget_show (aw->priv->entry);
  gtk_box_pack_start (GTK_BOX (vbox), aw->priv->entry, FALSE, FALSE, 0);
  gtk_entry_set_visibility (GTK_ENTRY (aw->priv->entry), FALSE);

  /* VTE terminal */
  aw->priv->vte = vte_terminal_new ();
  gtk_widget_hide (aw->priv->vte);
  gtk_box_pack_start (GTK_BOX (dlg_root), aw->priv->vte, FALSE, FALSE, 0);
	
  /* buttons panel */
  action_area = gtk_hbutton_box_new ();
  gtk_widget_show (action_area);
  gtk_box_pack_start (GTK_BOX (dlg_root), action_area, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area), GTK_BUTTONBOX_END);

  /* cancel button */
  aw->priv->btn_cancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (aw->priv->btn_cancel);
  gtk_container_add (GTK_CONTAINER (action_area), aw->priv->btn_cancel);
  GTK_WIDGET_SET_FLAGS (aw->priv->btn_cancel, GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (aw->priv->btn_cancel), "clicked", G_CALLBACK (on_btn_cancel_clicked_cb), aw);

  /* OK button */
  aw->priv->btn_ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (aw->priv->btn_ok);
  gtk_container_add (GTK_CONTAINER (action_area), aw->priv->btn_ok);
  GTK_WIDGET_SET_FLAGS (aw->priv->btn_ok, GTK_CAN_DEFAULT|GTK_HAS_DEFAULT);
  g_signal_connect (G_OBJECT (aw->priv->btn_ok), "clicked", G_CALLBACK (on_btn_ok_clicked_cb), aw);
  
  // FIXME: Why doesn't activate default work? This sure is an ugly hack.
  // gtk_entry_set_activates_default (GTK_ENTRY (aw->priv->entry), TRUE);
  g_signal_connect (G_OBJECT (aw->priv->entry), "activate", G_CALLBACK (on_btn_ok_clicked_cb), aw);
	
  return;
}

GtkWidget *
gpt_auth_window_new (GtkWindow *w)
{
  GptAuthWindow *aw;
  aw = g_object_new (GPT_AUTH_WINDOW_TYPE, "type", GTK_WINDOW_TOPLEVEL, NULL);
	
  gtk_window_set_transient_for (GTK_WINDOW (aw), w);
	
  return (GTK_WIDGET (aw));
}

static gboolean
on_timeout_cb (GptAuthWindow *aw)
{
  const gchar *passwd = gtk_entry_get_text (GTK_ENTRY (aw->priv->entry));
  gchar *str;
	
  if (aw->priv->pid)
    {
      str = vte_terminal_get_text_range(VTE_TERMINAL(aw->priv->vte),
                                        0, 0, 0, 16,
                                        vte_is_selected,
                                        NULL, NULL);
		
      if (str != NULL)
        {
          if (strncmp (&str[1], "assword:", strlen ("assword:")) == 0)
            {
              /* FIXME: it may be "root's password:" as well */
              vte_terminal_feed_child (VTE_TERMINAL (aw->priv->vte),
                                       passwd, strlen (passwd));
              vte_terminal_feed_child (VTE_TERMINAL (aw->priv->vte),
                                       "\n", 1);
              aw->priv->auth_done = TRUE;
              the_app->root_password = g_strdup (passwd);
              return FALSE;
            }
			
          else
            {
              aw->priv->auth_done = TRUE; /* failed to run su */
              return FALSE;
            }
          g_free (str);
        }
    }
	
  return TRUE;
}

static void on_btn_ok_clicked_cb (GtkWidget *widget, GptAuthWindow *aw)
{
  VteReaper *reaper;
  gchar *args[4] = { "su", "-c", "sleep 1 && exit 0", NULL };
	
  gtk_widget_set_sensitive (aw->priv->btn_ok, FALSE);
  gtk_widget_set_sensitive (aw->priv->btn_cancel, FALSE);
  gtk_widget_set_sensitive (aw->priv->entry, FALSE);
	
  reaper = vte_reaper_get ();
  g_signal_connect (G_OBJECT (reaper),
                    "child-exited",
                    G_CALLBACK (on_child_exited_cb),
                    aw);
  aw->priv->auth_done = FALSE;
  
  aw->priv->pid = vte_terminal_fork_command (VTE_TERMINAL (aw->priv->vte),
                                             args[0], args,
                                             NULL, NULL,
                                             TRUE, TRUE, TRUE);  
	
  g_timeout_add (200, (gpointer) on_timeout_cb, aw);
}

static void
on_btn_cancel_clicked_cb (GtkWidget *widget, GptAuthWindow *aw)
{
  the_app->need_auth = FALSE;
	
  g_object_set (the_app->main_window, "browse-only", TRUE, NULL);
	
  gtk_widget_destroy (GTK_WIDGET (aw));
}

static void
on_child_exited_cb (VteReaper *reaper, gint child_pid, gint retval, GptAuthWindow *aw)
{
  g_return_if_fail (GPT_IS_AUTH_WINDOW (aw));
  
  if (retval == 0) /* authentication succeeded */
      the_app->need_auth = TRUE;
  
  else
    {
      gpt_warning (GTK_WINDOW (aw), "Authentication failed");
      g_object_set (the_app->main_window, "browse-only", TRUE, NULL);
      the_app->need_auth = FALSE;
    }
	
    g_signal_handlers_disconnect_by_func (G_OBJECT (reaper),
                                         G_CALLBACK (on_child_exited_cb),
                                         aw);
    gtk_widget_destroy (GTK_WIDGET (aw));
}
