/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #include <gtk/gtk.h>

#include "configure_window.h"
#include "configure_cell_renderer.h"

static void
gpt_configure_window_class_init (GptConfigureWindowClass *klass);

static void
gpt_configure_window_init (GptConfigureWindow *cw);
static void
gpt_configure_window_set_property (GObject                *obj,
                                   guint                  prop_id,
                                   const GValue           *val,
                                   GParamSpec             *pspec);
static void
on_btn_ok_clicked_cb              (GtkWidget              *widget,
                                   GptConfigureWindow     *cw);
static void
on_btn_cancel_clicked_cb          (GtkWidget              *widget,
                                   GptConfigureWindow     *cw);
static gboolean
on_window_delete_event_cb         (GtkWidget              *widget,
                                   GdkEvent               *event,
                                   gpointer               data);
static void
on_option_cell_changed_cb         (ConfigureCellRenderer  *cell,
                                   const gchar            *path_str,
                                   gchar                  *new_value,
                                   GptConfigureWindow     *cw);
static gboolean
on_option_cell_check_writable_cb  (ConfigureCellRenderer *cell,
                                   const gchar           *path_str,
                                   GptConfigureWindow    *cw);


static GtkWindowClass *parent_class = NULL;

enum {
  PROP_OPTIONS = 1,
};

enum
{
  COL_OPT_NAME = 0,     /* option name (--with-foo) */
  COL_OPT_TYPE,         /* option value type (G_TYPE_STRING/G_TYPE_BOOLEAN) */
  COL_OPT_VALUE,        /* option value (gchar *; "\0"=FALSE, any other string=TRUE for boolean values) */
  COL_OPT_DESC,         /* option meaning (Enable libFoo support) */
  COL_OPT_MODIFIED,     /* whether user has modified this option */
  NUM_OPT_COLS
};

GType
gpt_configure_window_get_type (void)
{
  static GType type = 0;

  if (type == 0)
    {
      static const GTypeInfo info =
        {
          sizeof (GptConfigureWindowClass),
          NULL,   /* base_init */
          NULL,   /* base_finalize */
          (GClassInitFunc) gpt_configure_window_class_init,   /* class_init */
          NULL,   /* class_finalize */
          NULL,   /* class_data */
          sizeof (GptConfigureWindow),
          0,      /* n_preallocs */
          (GInstanceInitFunc) gpt_configure_window_init    /* instance_init */
        };
		
      type = g_type_register_static (GTK_TYPE_WINDOW, "GptConfigureWindow", &info, 0);
    }
  return type;
}

static void
gpt_configure_window_class_init (GptConfigureWindowClass *klass)
{
  GObjectClass *obj_class;
  obj_class = G_OBJECT_CLASS (klass);
	
  parent_class = (GtkWindowClass *) g_type_class_peek_parent (klass);

  obj_class->set_property = gpt_configure_window_set_property;

  g_object_class_install_property (obj_class,
            PROP_OPTIONS,
            g_param_spec_pointer ("options", /* name */
                                  "Options", /* nick */
                                  "Configuration Options", /* blurb */
                                  G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)); /* options */
	
}

static void
gpt_configure_window_init (GptConfigureWindow *cw)
{
  g_assert (GPT_IS_CONFIGURE_WINDOW (cw));
	
  GtkWidget *vbox9;
  GtkWidget *hbuttonbox2;
  GtkWidget *btn_ok;
  GtkWidget *btn_cancel;
  GtkWidget *scrolledwindow6;
	
  gtk_window_set_title (GTK_WINDOW (cw), "Configure Options");
	
  cw->priv = g_new0 (GptConfigureWindowPriv, 1);
  g_return_if_fail (cw->priv != NULL);

  gtk_window_set_modal (GTK_WINDOW (cw), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (cw), GDK_WINDOW_TYPE_HINT_DIALOG);
  
  gtk_container_set_border_width (GTK_CONTAINER (cw), 12);

  /* main vbox */
  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_box_set_spacing (GTK_BOX (vbox9), 12);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (cw), vbox9);
	
  /* the button box */
  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox2);
  gtk_box_pack_end (GTK_BOX (vbox9), hbuttonbox2, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox2), GTK_BUTTONBOX_END);

  /* the OK button*/
  btn_ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (btn_ok);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_ok);
  GTK_WIDGET_SET_FLAGS (btn_ok, GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (btn_ok),
                    "clicked",
                    G_CALLBACK (on_btn_ok_clicked_cb),
                    cw);

  /* the CANCEL button */
  btn_cancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (btn_cancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_cancel);
  GTK_WIDGET_SET_FLAGS (btn_cancel, GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (btn_cancel),
                    "clicked",
                    G_CALLBACK (on_btn_cancel_clicked_cb),
                    cw);

  /* the scroll window of the treeview */
  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (vbox9), scrolledwindow6, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);

  /* the treeview */
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *col;
  cw->priv->treeview = gtk_tree_view_new ();
  gtk_widget_show (cw->priv->treeview);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), cw->priv->treeview);
	
  g_signal_connect (G_OBJECT (cw),
                    "delete-event",
                    G_CALLBACK (on_window_delete_event_cb),
                    NULL);

  /* the treeview store */
  cw->priv->store = gtk_list_store_new (NUM_OPT_COLS,
                                        G_TYPE_STRING,  /* option */
                                        G_TYPE_INT,     /* value type */
                                        G_TYPE_STRING,  /* value */
                                        G_TYPE_STRING,  /* description */
                                        G_TYPE_BOOLEAN); /* modified */

  renderer = gtk_cell_renderer_text_new ();
  col = gtk_tree_view_column_new_with_attributes ("Option",
                                                  renderer,
                                                  "text", COL_OPT_DESC,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (cw->priv->treeview), col);

  renderer = configure_cell_renderer_new ();
  col = gtk_tree_view_column_new_with_attributes ("Value",
                                                  renderer,
                                                  "type", COL_OPT_TYPE,
                                                  "value", COL_OPT_VALUE,
                                                  "modified", COL_OPT_MODIFIED,
                                                  NULL);
  g_signal_connect (G_OBJECT (renderer),
                    "changed",
                    G_CALLBACK (on_option_cell_changed_cb),
                    cw);
  g_signal_connect (G_OBJECT (renderer),
                    "check_writable",
                    G_CALLBACK (on_option_cell_check_writable_cb),
                    cw);

  gtk_tree_view_append_column (GTK_TREE_VIEW (cw->priv->treeview), col);
  
  gtk_tree_view_set_model (GTK_TREE_VIEW (cw->priv->treeview),
                           GTK_TREE_MODEL (cw->priv->store));
}

GtkWidget *
gpt_configure_window_new (GList *list)
{
	return GTK_WIDGET (g_object_new (GPT_CONFIGURE_WINDOW_TYPE, "type", GTK_WINDOW_TOPLEVEL, "options",list,NULL));
}

static void
gpt_configure_window_set_property (GObject      *obj,
                                   guint        prop_id,
                                   const GValue *val,
                                   GParamSpec *pspec)
{
  g_return_if_fail (GPT_IS_CONFIGURE_WINDOW (obj));
	
  GList *the_list;
  GtkTreeIter iter;
  GptConfigureOptions *opt;
	
  switch (prop_id)
    {
      case PROP_OPTIONS:
        the_list = (GList *) g_value_get_pointer(val);

        while (the_list != NULL)
          {
            opt = (GptConfigureOptions *) the_list->data;
            gtk_list_store_append (GPT_CONFIGURE_WINDOW (obj)->priv->store, &iter);
            gtk_list_store_set (GPT_CONFIGURE_WINDOW (obj)->priv->store, &iter,
                                COL_OPT_NAME, opt->name,
                                COL_OPT_TYPE, opt->type,
                                COL_OPT_VALUE, opt->value,
                                COL_OPT_DESC, opt->desc,
                                COL_OPT_MODIFIED, FALSE,
                                -1);
            the_list = the_list->next;
          }
			
        break;
          
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
        break;
    }
}

static void
on_btn_ok_clicked_cb (GtkWidget *widget, GptConfigureWindow *cw)
{
	g_return_if_fail (GPT_IS_CONFIGURE_WINDOW (cw));
}

static void
on_btn_cancel_clicked_cb (GtkWidget *widget, GptConfigureWindow *cw)
{
  g_return_if_fail (GPT_IS_CONFIGURE_WINDOW (cw));
  gtk_widget_destroy (GTK_WIDGET (cw));
}

static gboolean
on_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  g_return_val_if_fail (GPT_IS_CONFIGURE_WINDOW (widget), TRUE);
  gtk_widget_destroy (widget);
  
  return TRUE;
}

static void
on_option_cell_changed_cb (ConfigureCellRenderer *cell,
                           const gchar           *path_str,
                           gchar                 *new_value,
                           GptConfigureWindow    *cw)
{
  GtkTreeIter iter;
  GtkTreePath *path;

  path = gtk_tree_path_new_from_string (path_str);
  gtk_tree_model_get_iter (GTK_TREE_MODEL(cw->priv->store), &iter, path);

  gtk_list_store_set (cw->priv->store, &iter,
                      COL_OPT_VALUE, new_value,
                      COL_OPT_MODIFIED, TRUE,
                      -1);
}

static gboolean
on_option_cell_check_writable_cb (ConfigureCellRenderer *cell,
                                  const gchar           *path_str,
                                  GptConfigureWindow    *cw)
{
  GtkTreeIter iter;
  GtkTreePath *path;
  GType type;

  path = gtk_tree_path_new_from_string (path_str);
  gtk_tree_model_get_iter (GTK_TREE_MODEL(cw->priv->store), &iter, path);
  gtk_tree_model_get (GTK_TREE_MODEL (cw->priv->store), &iter, COL_OPT_TYPE, &type, -1);

  if (type == G_TYPE_STRING)
      return TRUE;
  
  else if (type != G_TYPE_BOOLEAN)
      g_warning ("Unhandled type: %d", (int) type);
  
  return FALSE;
}
