/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "gpt.h"

static void gpt_app_class_init (GptAppClass *klass);
static void gpt_app_init (GptApp *app);
/* static void gpt_app_finalize (GObject *object); */
static void gpt_app_set_property (GObject       *obj,
                                  guint         prop_id,
                                  const GValue  *val,
                                  GParamSpec    *pspec);
static void gpt_app_get_property (GObject       *obj,
                                  guint         prop_id,
                                  GValue        *val,
                                  GParamSpec    *pspec);

enum {
  PROP_MAIN_WINDOW_WIDTH = 1,
  PROP_MAIN_WINDOW_HEIGHT,
  PROP_MAIN_WINDOW_PANED_POSITION,
  PROP_DETAILS_WINDOW_WIDTH,
  PROP_DETAILS_WINDOW_HEIGHT,
  PROP_INSTALL_PACKAGE_DIR,
  PROP_BUILD_PACKAGE_DIR,
  PROP_PROGRESS_WINDOW_WIDTH,
  PROP_PROGRESS_WINDOW_HEIGHT,
  PROP_OPTIMIZATION,
  PROP_PATCH_DIR,
};

#define GCONF_PATH                      "/apps/gnome-pkgtool"
#define KEY_MAIN_WINDOW_WIDTH           GCONF_PATH"/ui/main_window_width"
#define KEY_MAIN_WINDOW_HEIGHT          GCONF_PATH"/ui/main_window_height"
#define KEY_MAIN_WINDOW_PANED_POSITION  GCONF_PATH"/ui/main_window_paned_position"
#define KEY_DETAILS_WINDOW_WIDTH        GCONF_PATH"/ui/details_window_width"
#define KEY_DETAILS_WINDOW_HEIGHT       GCONF_PATH"/ui/details_window_height"
#define KEY_INSTALL_PACKAGE_DIR         GCONF_PATH"/install_package_dir"
#define KEY_BUILD_PACKAGE_DIR           GCONF_PATH"/build_package_dir"
#define KEY_PROGRESS_WINDOW_WIDTH       GCONF_PATH"/ui/progress_window_width"
#define KEY_PROGRESS_WINDOW_HEIGHT      GCONF_PATH"/ui/progress_window_height"
#define KEY_OPTIMIZATION                GCONF_PATH"/optimization"
#define KEY_PATCH_DIR                   GCONF_PATH"/patch_dir"

static GObjectClass *parent_class = NULL;

GType
gpt_app_get_type (void)
{
  static GType type = 0;

  if (type == 0)
    {
      static const GTypeInfo info =
        {
          sizeof (GptAppClass),
          NULL,   /* base_init */
          NULL,   /* base_finalize */
          (GClassInitFunc) gpt_app_class_init,   /* class_init */
          NULL,   /* class_finalize */
          NULL,   /* class_data */
          sizeof (GptApp),
          0,      /* n_preallocs */
          (GInstanceInitFunc) gpt_app_init    /* instance_init */
        };
		
      type = g_type_register_static (G_TYPE_OBJECT, "GptApp", &info, 0);
    }
  return type;
}

static void
gpt_app_class_init (GptAppClass *klass)
{
  GObjectClass *object_class;

  parent_class = (GObjectClass*) g_type_class_peek_parent (klass);
  object_class = (GObjectClass*) klass;

  /* object_class->finalize = gpt_app_finalize; */
  
  object_class->set_property = gpt_app_set_property;
  object_class->get_property = gpt_app_get_property;
  
  g_object_class_install_property (object_class,
                                   PROP_MAIN_WINDOW_WIDTH,
                                   g_param_spec_int ("main-window-width",
                                                     "Main Window width",
                                                     "Main Window height",
                                                     0, 4096, 1,
                                                     G_PARAM_READWRITE));

  g_object_class_install_property (object_class,
                                   PROP_MAIN_WINDOW_HEIGHT,
                                   g_param_spec_int ("main-window-height",
                                                     "Main Window height",
                                                     "Main Window height",
                                                     0, 4096, 1,
                                                     G_PARAM_READWRITE));
														
  g_object_class_install_property (object_class,
                                   PROP_MAIN_WINDOW_PANED_POSITION,
                                   g_param_spec_int ("main-window-paned-position",
                                                "Main Window paned position",
                                                "Main Window paned position",
                                                0, 4096, 1,
                                                G_PARAM_READWRITE));
                                                                                                          
  g_object_class_install_property (object_class,
                                   PROP_DETAILS_WINDOW_WIDTH,
                                   g_param_spec_int ("details-window-width",
                                                     "Details Window width",
                                                     "Details Window width",
                                                     0, 4096, 1,
                                                     G_PARAM_READWRITE));

  g_object_class_install_property (object_class,
                                   PROP_DETAILS_WINDOW_HEIGHT,
                                   g_param_spec_int ("details-window-height",
                                                     "Details Window height",
                                                     "Details Window height",
                                                     0, 4096, 1,
                                                     G_PARAM_READWRITE));
                                                                                                          
  g_object_class_install_property (object_class,
                                   PROP_PROGRESS_WINDOW_WIDTH,
                                   g_param_spec_int ("progress-window-width",
                                                     "Progress Window width",
                                                     "Progress Window width when expanded",
                                                     0, 4096, 1,
                                                     G_PARAM_READWRITE));
                                                                                                          
  g_object_class_install_property (object_class,
                                   PROP_PROGRESS_WINDOW_HEIGHT,
                                   g_param_spec_int ("progress-window-height",
                                                     "Progress Window height",
                                                     "Progress Window height when expanded",
                                                     0, 4096, 1,
                                                     G_PARAM_READWRITE));
                                                                                                          
  g_object_class_install_property (object_class,
                                   PROP_INSTALL_PACKAGE_DIR,
                                   g_param_spec_string ("install-package-dir",
                              "Install Package directory",
                              "Install Package filechooser default directory",
                              "~",
                              G_PARAM_READWRITE));
                                                                                                          
  g_object_class_install_property (object_class,
                                   PROP_BUILD_PACKAGE_DIR,
                                   g_param_spec_string ("build-package-dir",
                                "Build Package directory",
                                "Build Package filechooser default directory",
                                "~",
                                G_PARAM_READWRITE));
                                                                                                          
  g_object_class_install_property (object_class,
                                   PROP_OPTIMIZATION,
                                   g_param_spec_string ("optimization",
                                                       "Optimization",
                                                       "Default optimization",
                                                       "none",
                                                       G_PARAM_READWRITE));
  g_object_class_install_property (object_class,
                                   PROP_PATCH_DIR,
                                   g_param_spec_string ("patch-dir",
                                                        "Patch directory",
                                                        "Add Patch filechooser default directory",
                                                        "~",
                                                        G_PARAM_READWRITE));
}

static void
gpt_app_init (GptApp *app)
{
	g_assert (app != NULL);
	g_assert (GPT_IS_APP (app));
	
	app->client = gconf_client_get_default ();
}

static void
gpt_app_set_property (GObject *obj, guint prop_id, const GValue *val, GParamSpec *pspec)
{
	g_return_if_fail (GPT_IS_APP (obj));
	GptApp *app = GPT_APP (obj);
	
	gchar *key = NULL;
	gint type;
	
	switch (prop_id) {
		case PROP_MAIN_WINDOW_HEIGHT:
			key = g_strdup (KEY_MAIN_WINDOW_HEIGHT);
			type = GCONF_VALUE_INT;
			break;
		case PROP_MAIN_WINDOW_WIDTH:
			key = g_strdup (KEY_MAIN_WINDOW_WIDTH);
			type = GCONF_VALUE_INT;
			break;
		case PROP_MAIN_WINDOW_PANED_POSITION:
			key = g_strdup (KEY_MAIN_WINDOW_PANED_POSITION);
			type = GCONF_VALUE_INT;
			break;
		case PROP_DETAILS_WINDOW_HEIGHT:
			key = g_strdup (KEY_DETAILS_WINDOW_HEIGHT);
			type = GCONF_VALUE_INT;
			break;
		case PROP_DETAILS_WINDOW_WIDTH:
			key = g_strdup (KEY_DETAILS_WINDOW_WIDTH);
			type = GCONF_VALUE_INT;
			break;
		case PROP_PROGRESS_WINDOW_HEIGHT:
			key = g_strdup (KEY_PROGRESS_WINDOW_HEIGHT);
			type = GCONF_VALUE_INT;
			break;
		case PROP_PROGRESS_WINDOW_WIDTH:
			key = g_strdup (KEY_PROGRESS_WINDOW_WIDTH);
			type = GCONF_VALUE_INT;
			break;
		case PROP_BUILD_PACKAGE_DIR:
			key = g_strdup (KEY_BUILD_PACKAGE_DIR);
			type = GCONF_VALUE_STRING;
			break;
		case PROP_INSTALL_PACKAGE_DIR:
			key = g_strdup (KEY_INSTALL_PACKAGE_DIR);
			type = GCONF_VALUE_STRING;
			break;
		case PROP_OPTIMIZATION:
			key = g_strdup (KEY_OPTIMIZATION);
			type = GCONF_VALUE_STRING;
			break;
		case PROP_PATCH_DIR:
			key = g_strdup (KEY_PATCH_DIR);
			type = GCONF_VALUE_STRING;
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
			return;
			break;
	}
	
	if (key) {
		GError *err = NULL;
		
		switch (type) {
			case GCONF_VALUE_STRING:
				gconf_client_set_string (app->client, key, g_value_get_string (val), &err);
				break;
			case GCONF_VALUE_INT:
				gconf_client_set_int (app->client, key, g_value_get_int (val), &err);
				break;
			default:
				g_assert (type);
				break;
		}
		
		if (err)
			g_warning (err->message);
		g_free (err);
	}		
	
	g_free (key);
}

static void
gpt_app_get_property (GObject *obj, guint prop_id, GValue *val, GParamSpec *pspec)
{
	g_return_if_fail (GPT_IS_APP (obj));
	GptApp *app = GPT_APP (obj);
	
	gchar *key = NULL;
	gint type;
	
	switch (prop_id) {
		case PROP_MAIN_WINDOW_HEIGHT:
			key = g_strdup (KEY_MAIN_WINDOW_HEIGHT);
			type = GCONF_VALUE_INT;
			break;
		case PROP_MAIN_WINDOW_WIDTH:
			key = g_strdup (KEY_MAIN_WINDOW_WIDTH);
			type = GCONF_VALUE_INT;
			break;
		case PROP_MAIN_WINDOW_PANED_POSITION:
			key = g_strdup (KEY_MAIN_WINDOW_PANED_POSITION);
			type = GCONF_VALUE_INT;
			break;
		case PROP_DETAILS_WINDOW_HEIGHT:
			key = g_strdup (KEY_DETAILS_WINDOW_HEIGHT);
			type = GCONF_VALUE_INT;
			break;
		case PROP_DETAILS_WINDOW_WIDTH:
			key = g_strdup (KEY_DETAILS_WINDOW_WIDTH);
			type = GCONF_VALUE_INT;
			break;
		case PROP_PROGRESS_WINDOW_HEIGHT:
			key = g_strdup (KEY_PROGRESS_WINDOW_HEIGHT);
			type = GCONF_VALUE_INT;
			break;
		case PROP_PROGRESS_WINDOW_WIDTH:
			key = g_strdup (KEY_PROGRESS_WINDOW_WIDTH);
			type = GCONF_VALUE_INT;
			break;
		case PROP_BUILD_PACKAGE_DIR:
			key = g_strdup (KEY_BUILD_PACKAGE_DIR);
			type = GCONF_VALUE_STRING;
			break;
		case PROP_INSTALL_PACKAGE_DIR:
			key = g_strdup (KEY_INSTALL_PACKAGE_DIR);
			type = GCONF_VALUE_STRING;
			break;
		case PROP_OPTIMIZATION:
			key = g_strdup (KEY_OPTIMIZATION);
			type = GCONF_VALUE_STRING;
			break;
		case PROP_PATCH_DIR:
			key = g_strdup (KEY_PATCH_DIR);
			type = GCONF_VALUE_STRING;
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
			return;
			break;
	}
	
	if (key) {
		GError *err = NULL;
		gchar *str;
		gint num;
		
		switch (type) {
			case GCONF_VALUE_STRING:
				str = gconf_client_get_string (app->client, key, &err);
				if (err) {
					g_warning (err->message);
					g_free (err);
					g_value_set_string (val, "");
				}
				else {
					g_value_set_string (val, str);
					g_free (str);
				}
				break;
			case GCONF_VALUE_INT:
				num = gconf_client_get_int (app->client, key, &err);
				if (err) {
					g_warning (err->message);
					g_free (err);
					g_value_set_int (val, 0);
				}
				else 
					g_value_set_int (val, num);
				break;
			default:
				g_assert (type);
				break;
		}
	}		
	
	g_free (key);
}
