/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <gnome.h>
#include <libgnomeui/libgnomeui.h>
#include <config.h>
#include "main_window.h"
#include "packages.h"
#include "details_window.h"
#include "progress_window.h"
#include "build_window.h"
#include "pkglist.h"
#include "gpt.h"

static void gpt_main_window_class_init (GptMainWindowClass *klass);
static void gpt_main_window_init (GptMainWindow *mw);
static void gpt_main_window_set_property (GObject       *obj,
                                          guint         prop_id,
                                          const         GValue *val,
                                          GParamSpec    *pspec);


static gboolean on_menu_close_cb (GtkWidget     *widget,
                                  GptMainWindow *mw);
static gboolean on_menu_about_cb (GtkWidget     *widget,
                                  GptMainWindow *mw);
static gboolean on_menu_build_cb (GtkWidget     *widget,
                                  GptMainWindow *mw);
static gboolean on_menu_install_cb (GtkWidget           *widget,
                                    GptMainWindow       *mw);
static gboolean on_menu_refresh_cb (GtkWidget           *widget,
                                    GptMainWindow       *mw);

static void on_btn_close_clicked_cb (GtkWidget          *widget,
                                     GptMainWindow      *mw);
static void on_btn_details_clicked_cb (GtkWidget        *widget,
                                       GptMainWindow    *mw);
static void on_btn_uninstall_clicked_cb (GtkWidget      *widget,
                                         GptMainWindow  *mw);

static gboolean on_window_delete_event_cb (GtkWidget    *widget,
                                           GdkEvent     *event,
                                           gpointer data);
static gboolean on_window_configure_event_cb (GtkWidget *widget,
                                              GdkEvent  *event,
                                              gpointer data);
static gboolean
on_packages_list_selection_changed_cb (GtkTreeSelection *selection,
                                       GptMainWindow    *mw);

static GtkWindowClass *parent_class = NULL;

extern GptApp *the_app;

enum {
	PROP_BROWSE_ONLY = 1,
};

GType
gpt_main_window_get_type (void)
{
  static GType type = 0;

  if (type == 0)
    {
      static const GTypeInfo info =
        {
          sizeof (GptMainWindowClass),
          NULL,   /* base_init */
          NULL,   /* base_finalize */
          (GClassInitFunc) gpt_main_window_class_init,   /* class_init */
          NULL,   /* class_finalize */
          NULL,   /* class_data */
          sizeof (GptMainWindow),
          0,      /* n_preallocs */
          (GInstanceInitFunc) gpt_main_window_init    /* instance_init */
        };
		
      type = g_type_register_static (GTK_TYPE_WINDOW,
                                     "GptMainWindow",
                                     &info,
                                     0);
    }
  return type;
}

static void
gpt_main_window_class_init (GptMainWindowClass *klass)
{
  GObjectClass *obj_class = (GObjectClass *) klass;
  parent_class = (GtkWindowClass*) g_type_class_peek_parent (klass);
	
  obj_class->set_property = gpt_main_window_set_property;
	
  g_object_class_install_property (obj_class,
                                   PROP_BROWSE_ONLY,
                                   g_param_spec_boolean ("browse-only",
                                                         "Browse only",
                                                         "Browse only",
                                                         FALSE,
                                                         G_PARAM_WRITABLE));
}

static void
gpt_main_window_set_property (GObject           *obj,
                              guint             prop_id,
                              const GValue      *val,
                              GParamSpec        *pspec)
{
  GptMainWindow *mw  = GPT_MAIN_WINDOW (obj);

  switch (prop_id)
    {
      case PROP_BROWSE_ONLY:
        if (g_value_get_boolean (val))
          {
            gtk_widget_set_sensitive (mw->priv->menu_install, FALSE);
            gtk_widget_set_sensitive (mw->priv->menu_build, FALSE);
            gtk_widget_set_sensitive (mw->priv->btn_uninstall, FALSE);
          }
        else
          {
            gtk_widget_set_sensitive (mw->priv->menu_install, TRUE);
            gtk_widget_set_sensitive (mw->priv->menu_build, TRUE);
            gtk_widget_set_sensitive (mw->priv->btn_uninstall, TRUE);
          }
        break;
          
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
        break;
    }
}

static void
gpt_main_window_init (GptMainWindow *mw)
{
  g_assert (mw != NULL);
  g_assert (GPT_IS_MAIN_WINDOW (mw));
	
  mw->priv = g_new0 (GptMainWindowPriv, 1);
  g_assert (mw->priv != NULL);
	
  GtkWidget *vbox1;
  GtkWidget *align;
  GtkWidget *scrolledwindow1;
  GtkWidget *hbox31;
  GtkWidget *hbox32;
  GtkWidget *scrolledwindow3;
  GtkWidget *viewport1;
  GtkWidget *hbox3;
  GtkWidget *hbox5;
  GtkWidget *alignment2;
  GtkWidget *hbox6;
  GtkWidget *image5;
  GtkWidget *label3;
  GtkWidget *alignment3;
  GtkWidget *hbox7;
  GtkWidget *image6;
  GtkWidget *label4;
  GtkWidget *hbox4;
	
  GnomeUIInfo menu_file_uiinfo[] =
    {
      { GNOME_APP_UI_ITEM, "_Install Package...",
        "Install a new TGZ package",
        (gpointer) on_menu_install_cb, (gpointer) mw, NULL,
        GNOME_APP_PIXMAP_STOCK, "gtk-add",
        0, (GdkModifierType) 0, NULL },
      { GNOME_APP_UI_ITEM, "_Build From Source...",
	"Build package from source tarball",
	(gpointer) on_menu_build_cb, (gpointer) mw, NULL,
	GNOME_APP_PIXMAP_STOCK, "gtk-convert",
	0, (GdkModifierType) 0, NULL },
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_MENU_CLOSE_ITEM (on_menu_close_cb, NULL),
      GNOMEUIINFO_END
    };
    
  GnomeUIInfo menu_view_uiinfo[] =
    {
      { GNOME_APP_UI_ITEM, "_Refresh",
        "Refresh packages list",
        (gpointer) on_menu_refresh_cb, (gpointer) mw, NULL,
        GNOME_APP_PIXMAP_STOCK, "gtk-refresh",
        0, (GdkModifierType) 0, NULL },
      GNOMEUIINFO_END
    };
    
  GnomeUIInfo menu_help_uiinfo[] =
    {
      GNOMEUIINFO_MENU_ABOUT_ITEM (on_menu_about_cb, (gpointer) mw),
      GNOMEUIINFO_END
    };
    
  GnomeUIInfo menubar_uiinfo[] =
    {
      GNOMEUIINFO_MENU_FILE_TREE (menu_file_uiinfo),
      GNOMEUIINFO_MENU_VIEW_TREE (menu_view_uiinfo),
      GNOMEUIINFO_MENU_HELP_TREE (menu_help_uiinfo),
      GNOMEUIINFO_END
    };

  gtk_window_set_title (GTK_WINDOW (mw), "Slackware Package Manager");
  gtk_window_set_icon_from_file(GTK_WINDOW(mw),
                                PACKAGE_PIXMAPS_DIR"/gnome-pkgtool.png",
                                NULL);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mw), vbox1);

  /* menu */
  mw->priv->menubar = gtk_menu_bar_new ();
  gtk_widget_show (mw->priv->menubar);
  gtk_box_pack_start (GTK_BOX (vbox1), mw->priv->menubar, FALSE, TRUE, 0);
  gnome_app_fill_menu (GTK_MENU_SHELL (mw->priv->menubar),
                       menubar_uiinfo,
                       NULL, FALSE, 0);
	
  /* remember pointers to some menu items */
  mw->priv->menu_install = menu_file_uiinfo[0].widget;
  mw->priv->menu_build = menu_file_uiinfo[1].widget;

  /* paned */
  align = gtk_alignment_new (0.5, 0.7, 1, 1);
  gtk_widget_show (align);
  gtk_box_pack_start (GTK_BOX (vbox1), align, TRUE, TRUE, 0);
  mw->priv->paned = gtk_vpaned_new ();
  gtk_widget_show (mw->priv->paned);
  gtk_container_add (GTK_CONTAINER (align), mw->priv->paned);
  gtk_paned_set_position (GTK_PANED (mw->priv->paned), 0);

  /* packages list */
  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1),
                                       GTK_SHADOW_IN);
  gtk_paned_pack1 (GTK_PANED (mw->priv->paned), scrolledwindow1, TRUE, FALSE);
  gtk_widget_set_size_request (scrolledwindow1, -1, 150);
  gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 6);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_ALWAYS);  

  mw->priv->packages_list = pkg_list_new ();
  g_assert (mw->priv->packages_list != NULL);

  gtk_widget_show (mw->priv->packages_list);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1),
                     mw->priv->packages_list);
  gtk_container_set_border_width (GTK_CONTAINER (mw->priv->packages_list), 1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (mw->priv->packages_list),
                                     FALSE);

  if (! packages_list_load (PKG_LIST (mw->priv->packages_list), NULL))
      g_error ("Cannot load packages list");

  GtkTreeSelection *selection;
  selection = gtk_tree_view_get_selection (
                                    GTK_TREE_VIEW (mw->priv->packages_list));
  g_signal_connect (G_OBJECT (selection),
                    "changed",
                    G_CALLBACK (on_packages_list_selection_changed_cb),
                    mw);

  /* can't remember what these two are for */
  hbox31 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox31);
  gtk_paned_pack2 (GTK_PANED (mw->priv->paned), hbox31, FALSE, TRUE);

  hbox32 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox31);
  gtk_box_pack_start (GTK_BOX (hbox31), hbox32, TRUE, TRUE, 0);
	
  /* selected package summary */
  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_box_pack_start (GTK_BOX (hbox31), scrolledwindow3, TRUE, TRUE, 0);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow3),
                                       GTK_SHADOW_IN);
  gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow3), 6);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);
  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), viewport1);
  gtk_container_set_border_width (GTK_CONTAINER (viewport1), 6);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport1), GTK_SHADOW_NONE);
  mw->priv->label_summary = gtk_label_new ("");
  gtk_widget_show (mw->priv->label_summary);
  gtk_container_add (GTK_CONTAINER (viewport1), mw->priv->label_summary);
  gtk_misc_set_alignment (GTK_MISC (mw->priv->label_summary), 0, 0);
  gtk_misc_set_padding (GTK_MISC (mw->priv->label_summary), 4, 0);

  /* buttons panel? */
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, TRUE, 0);

  hbox5 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (hbox3), hbox5, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox5), 12);

  /* uninstall button */
  mw->priv->btn_uninstall = gtk_button_new ();
  gtk_widget_show (mw->priv->btn_uninstall);
  gtk_box_pack_start (GTK_BOX (hbox5),
                      mw->priv->btn_uninstall,
                      FALSE, FALSE, 0);
  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (mw->priv->btn_uninstall), alignment2);
  hbox6 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox6);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox6);
  image5 = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image5);
  gtk_box_pack_start (GTK_BOX (hbox6), image5, FALSE, FALSE, 0);
  label3 = gtk_label_new_with_mnemonic ("Uninstall");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox6), label3, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (mw->priv->btn_uninstall),
                    "clicked",
                    G_CALLBACK (on_btn_uninstall_clicked_cb),
                    mw);

  /* details button */
  mw->priv->btn_details = gtk_button_new ();
  gtk_widget_show (mw->priv->btn_details);
  gtk_box_pack_start (GTK_BOX (hbox5),
                      mw->priv->btn_details,
                      FALSE, FALSE, 0);
  alignment3 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment3);
  gtk_container_add (GTK_CONTAINER (mw->priv->btn_details), alignment3);
  hbox7 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox7);
  gtk_container_add (GTK_CONTAINER (alignment3), hbox7);
  image6 = gtk_image_new_from_stock ("gtk-find", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image6);
  gtk_box_pack_start (GTK_BOX (hbox7), image6, FALSE, FALSE, 0);
  label4 = gtk_label_new_with_mnemonic ("Details...");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox7), label4, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (mw->priv->btn_details),
                    "clicked",
                    G_CALLBACK (on_btn_details_clicked_cb),
                    mw);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (hbox3), hbox4, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox4), 12);

  /* close button */
  mw->priv->btn_close = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (mw->priv->btn_close);
  gtk_box_pack_start (GTK_BOX (hbox4), mw->priv->btn_close, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (mw->priv->btn_close),
                    "clicked",
                    G_CALLBACK (on_btn_close_clicked_cb),
                    mw);
	
  gint w, h, pp;
  g_object_get (the_app, "main-window-width", &w, NULL);
  g_object_get (the_app, "main-window-height", &h, NULL);
  g_object_get (the_app, "main-window-paned-position", &pp, NULL);
	
  gtk_window_resize (GTK_WINDOW (mw), w, h);
	
  if (pp > 1)
      gtk_paned_set_position (GTK_PANED (mw->priv->paned), pp);
	
  g_signal_connect (G_OBJECT (mw),
                    "configure-event",
                    G_CALLBACK (on_window_configure_event_cb),
                    the_app);
  g_signal_connect (G_OBJECT (mw),
                    "delete-event",
                    G_CALLBACK (on_window_delete_event_cb),
                    the_app);
}

static gboolean
on_menu_build_cb (GtkWidget *widget, GptMainWindow *mw)
{
  GtkWidget *bw = gpt_build_window_new ();
  gtk_window_set_transient_for(GTK_WINDOW (bw), GTK_WINDOW (mw));
  gtk_window_set_destroy_with_parent (GTK_WINDOW (bw), TRUE);
  gtk_widget_show(bw);
  return TRUE;
}

static gboolean
on_menu_about_cb (GtkWidget *widget, GptMainWindow *mw)
{
  GtkWidget *dlg;
  GdkPixbuf *icon_pbuf;
  gchar *authors[3];
  authors[0] = "Andrei Yurkevich <urruru@ru.ru>";
  authors[1] = "Mark Pariente <markpariente@cmu.edu>";
  authors[2] = NULL;
  icon_pbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
                                       "gnome-pkgtool",
                                       48, 0, NULL);
  dlg = gnome_about_new("gnome-pkgtool",
                        VERSION,
                        "(c) 2004, Andrei Yurkevich",
                        "gnome-pkgtool is a Slackware Linux package manager for GTK+/GNOME",
                        (gpointer) authors,
                        NULL, NULL,
                        icon_pbuf);
  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(mw));
  gtk_window_set_type_hint (GTK_WINDOW (dlg), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_widget_show(dlg);
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
  return TRUE;
}

static gboolean
on_menu_refresh_cb (GtkWidget *widget, GptMainWindow *mw)
{
  packages_list_refresh (PKG_LIST (mw->priv->packages_list),
                         NULL);
  return TRUE;
}

static gboolean
on_menu_close_cb (GtkWidget *widget, GptMainWindow *mw)
{
  gtk_main_quit();
  
  return TRUE;
}

static gboolean
on_menu_install_cb (GtkWidget *widget, GptMainWindow *mw)
{
  g_return_val_if_fail (GPT_IS_MAIN_WINDOW (mw), TRUE);
  
  GtkWidget *chooser;
  gchar *path;
  chooser = gtk_file_chooser_dialog_new ("Install Package",
                                         GTK_WINDOW(mw),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         NULL);
  g_object_get (the_app, "install-package-dir", &path, NULL);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
                                       path);

  /* FIXME: set content-type (.TGZ) for file filtering */
  gchar *p;
  gchar *selected_filename;
  gchar *name;
	
  if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
    {
      selected_filename = gtk_file_chooser_get_filename (
                                                  GTK_FILE_CHOOSER (chooser));
      /* save selected path */
      path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (chooser));
      g_object_set (the_app, "install-package-dir", path, NULL);
      
      gtk_widget_destroy(chooser);
    }
  else
    {
      gtk_widget_destroy(chooser);
      return TRUE;
    }

  /* show upgrade/reinstall notification */
  p = rindex(selected_filename, '/');
  if(p)
      p++;
  name = g_strdup(p);
	
  if((p = strstr (name, ".tgz")) == NULL)
    {
      g_error ("package name does not end in .tgz");
      return TRUE;
    }
	
  /* check for previous version of package installed */
  gchar *tmp = strdup(name);
  gchar *cmd = NULL;
  if((p= rindex (tmp, '-'))) /* -1.tgz */
    {	
      *p = '\0';
      if((p=rindex (tmp, '-'))) /* -ix86-1.tgz */
        {
          *p = '\0';
          if((p=rindex (tmp, '-'))) /* -1.2.3-i386.tgz */
            {	
              *p = '\0';
              if (packages_list_find (PKG_LIST (mw->priv->packages_list),
                                        tmp))
                {
                  GtkWidget *dlg;
                  dlg = gtk_message_dialog_new (GTK_WINDOW (mw),
                                                GTK_DIALOG_MODAL,
                                                GTK_MESSAGE_WARNING,
                                                GTK_BUTTONS_OK_CANCEL,
                                                "Installing %s will upgrade previous version(s) of %s installed",
		                                name, tmp);
                  gint res = gtk_dialog_run(GTK_DIALOG(dlg));
                  gtk_widget_destroy(dlg);
                  g_free(tmp);
					
                  if (res == GTK_RESPONSE_OK) /* we're upgrading */
                    {
                      g_free (name);
                      cmd = g_strdup (PACKAGE_LIBEXEC_DIR"/gpt-upgrade");
                    }
                  else /* cancel */
                    {	
                      g_free (name);
                      g_free (selected_filename);
                      return TRUE;
                    }
                }
              else
                {
                  cmd = g_strdup ("installpkg");
                }
            }
        }
    }
	
  if(cmd == NULL)
      cmd = g_strdup("installpkg");
	
  /* prepare and run progress window */
  GtkWidget *pw;
  pw = gpt_progress_window_new ();
  g_return_val_if_fail (GPT_IS_PROGRESS_WINDOW (pw), TRUE);
	
  gchar *pw_cmd;
  pw_cmd = g_strdup_printf ("%s %s", cmd, selected_filename);
  
  if ((p = rindex (selected_filename, '/')) != NULL)
      name = g_strdup (++p);
  else
      name = strdup (selected_filename);
  
  p = name;
  
  if ((strncmp (&p[strlen (p) - 4], ".tgz", 4) == 0) && strlen (p) > 4);
      p[strlen (p) - 4] = '\0';
  
  gchar *str;
  str = g_strdup_printf ("<big><b>Package Installation</b></big>\n\nInstalling %s",
                         name);
  g_object_set (G_OBJECT (pw), "markup", str, NULL);
  g_free (str);
  
  str = g_strdup_printf ("Could not install package %s", name);
  g_object_set (pw, "failed-message", str, NULL);
  g_free (str);

  str = g_strdup_printf ("Package %s installed succesfully", name);
  g_object_set (pw, "complete-message", str, NULL);
  g_free (str);
  
  g_object_set (pw, "aborted-message", "Installation aborted", NULL);
  
  gtk_window_set_transient_for(GTK_WINDOW(pw), GTK_WINDOW(mw));
  gpt_progress_window_run (GPT_PROGRESS_WINDOW (pw), pw_cmd);
  
  g_free (selected_filename);
  g_free (pw_cmd);
  return TRUE;
}

static void
on_btn_uninstall_clicked_cb (GtkWidget *widget, GptMainWindow *mw)
{
  g_return_if_fail (GPT_IS_MAIN_WINDOW (mw));
	
  GtkTreeIter iter;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  PkgEntry *entry;
	
  selection = gtk_tree_view_get_selection (
                                     GTK_TREE_VIEW (mw->priv->packages_list));
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
      gtk_tree_model_get (model, &iter, 0, &entry, -1);
	
  if(IS_PKG_ENTRY (entry))
    {
      GptProgressWindow *pw;
      pw = GPT_PROGRESS_WINDOW (gpt_progress_window_new ());
      g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));

      gchar *cmd, *pkgname;
		
      g_object_get (entry, "package-name", &pkgname, NULL);
      cmd = g_strdup_printf ("/sbin/removepkg %s", pkgname);

      gchar *str = g_strdup_printf ("<big><b>Package Uninstall</b></big>\n\nRemoving %s",
                                    pkgname);
      g_object_set (G_OBJECT (pw), "markup", str, NULL);
      g_free (str);
    
      str = g_strdup_printf ("Could not uninstall package %s", pkgname);
      g_object_set (pw, "failed-message", str, NULL);
      g_free (str);
    
      str = g_strdup_printf ("Package %s uninstalled succesfully", pkgname);
      g_object_set (pw, "complete-message", str, NULL);
      g_free (str);

      g_object_set (pw, "aborted-message", "Uninstallation aborted", NULL);
      
		
      gtk_window_set_transient_for(GTK_WINDOW(pw), GTK_WINDOW(mw));
      gpt_progress_window_run (pw, cmd);
		
      g_free (cmd);
      g_object_unref (entry);
    }
}

static void
on_btn_details_clicked_cb (GtkWidget *widget, GptMainWindow *mw)
{
  GtkTreeIter iter;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  PkgEntry *entry;
	
  selection = gtk_tree_view_get_selection (
                                     GTK_TREE_VIEW (mw->priv->packages_list));
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
      gtk_tree_model_get (model, &iter, 0, &entry, -1);
	
  if(IS_PKG_ENTRY (entry))
    {
      GtkWidget *dw;
		
      dw = gpt_details_window_new(entry);
      g_assert (dw != NULL);
      g_assert (GPT_IS_DETAILS_WINDOW (dw));
		
      gtk_window_set_transient_for (GTK_WINDOW (dw), GTK_WINDOW (mw));
      gtk_widget_show (dw);
    
      g_object_unref (entry);
    }
}

static void
on_btn_close_clicked_cb (GtkWidget *widget, GptMainWindow *mw)
{
  gtk_main_quit();
}

static gboolean
on_packages_list_selection_changed_cb (GtkTreeSelection *selection,
                                       GptMainWindow    *mw)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  PkgEntry *entry;
	
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gtk_tree_model_get (model, &iter, 0, &entry, -1);
      if (IS_PKG_ENTRY (entry))
        {
          gchar *description;
          g_object_get (entry, "description", &description, NULL);
          gtk_label_set_markup (GTK_LABEL (mw->priv->label_summary),
                                description);
          g_object_unref (entry);
        }
    }
  
  return TRUE;
}

GtkWidget *
gpt_main_window_new (void)
{
  GptMainWindow *w;
  w = g_object_new (GPT_MAIN_WINDOW_TYPE,
                    "type", GTK_WINDOW_TOPLEVEL,
                    NULL);
  return (GTK_WIDGET (w));
}

static gboolean
on_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  g_return_val_if_fail (GPT_IS_MAIN_WINDOW (widget), FALSE);
	
  gtk_main_quit ();
	
  return TRUE;
}

static gboolean
on_window_configure_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  g_return_val_if_fail (GPT_IS_MAIN_WINDOW (widget), TRUE);
	
  gint w, h;	
  gtk_window_get_size (GTK_WINDOW (widget), &w, &h);
  g_object_set (the_app, "main-window-width", w, NULL);
  g_object_set (the_app, "main-window-height", h, NULL);
	
  gint pp;
  pp = gtk_paned_get_position (GTK_PANED (GPT_MAIN_WINDOW (widget)->priv->paned));
  g_object_set (the_app, "main-window-paned-position", pp, NULL);
	
  return FALSE;
}
